﻿using System.IO;
using CSCore;
using CSCore.Codecs;
using CSCore.CoreAudioAPI;
using CSCore.SoundOut;

namespace GameUtils.Audio.CsCore
{
    public class CSCoreEngine : AudioEngine
    {
        private ISoundOut CreateSoundOut(ref IWaveSource source)
        {
            ISoundOut soundOut;
            if (WasapiOut.IsSupportedOnCurrentPlatform)
                soundOut = new WasapiOut(true, AudioClientShareMode.Shared, 50);
            else
            {
                soundOut = new DirectSoundOut() { Latency = 100 };
                if (source.WaveFormat.BitsPerSample > 16)
                    source = source.ToSampleSource().ToWaveSource(16);
            }
            return soundOut;
        }

        public override AudioHandle CreateHandle(Stream stream)
        {
            IWaveSource source = CodecFactory.Instance.GetCodec(null, stream);
            ISoundOut soundOut = this.CreateSoundOut(ref source);

            return new CSCoreHandle(soundOut, source);
        }

        public override AudioHandle CreateHandle(string file)
        {
            IWaveSource source = CodecFactory.Instance.GetCodec(file);
            ISoundOut soundOut = this.CreateSoundOut(ref source);

            return new CSCoreHandle(soundOut, source);
        }
    }
}
