﻿using System;

namespace GameUtils.Input
{
    public abstract class InputFilter<TStateIn, TStateOut> : IInputListener<TStateIn>, IInputProvider<TStateOut>
    {
        bool IEngineComponent.IsCompatibleTo(IEngineComponent component)
        {
            return true;
        }

        public object Tag { get; set; }

        public virtual bool Active { get; set; }

        void IInputListener<TStateIn>.OnInputReceived(TStateIn state)
        {
            OnInputReceived(state);
        }

        protected abstract void OnInputReceived(TStateIn state);

        public abstract ListenerHandle<TStateOut> RegisterListener(IInputListener<TStateOut> listener);

        private bool disposed;

        public void Dispose()
        {
            if (!disposed)
            {
                Dispose(true);
                GC.SuppressFinalize(this);

                disposed = true;
            }
        }

        protected virtual void Dispose(bool disposing)
        { }

        ~InputFilter()
        {
            Dispose(false);
        }
    }
}
