﻿using GameUtils.Graphics.Data;

namespace GameUtils.Graphics
{
    public class SolidColorBrush : Brush<SolidColorBrushData>
    {
        Color4 color;
        bool colorChanged;
        readonly object locker;

        /// <summary>
        /// The color of this brush.
        /// </summary>
        public Color4 Color
        {
            get { return color; }
            set
            {
                lock (locker)
                {
                    color = value;
                    colorChanged = true;
                }
            }
        }

        public SolidColorBrush(Color4 color)
        {
            this.color = color;
            locker = new object();
        }

        protected override void ApplyChanges()
        {
            lock (locker)
            {
                if (colorChanged)
                {
                    Data.Color = color;
                    colorChanged = false;
                }
            }

            base.ApplyChanges();
        }

        protected override SolidColorBrushData CreateData(Factory factory)
        {
            lock (locker)
            {
                colorChanged = false;
                return factory.CreateSolidColorBrush(color);
            }
        }
    }
}
